#ifndef _CONFIGS_H_
#define _CONFIGS_H_

#include <limits>

const int minSizeofSample = 4; //  4 for homgography, 3 for affine.

namespace ProjectiveTransformation{
    enum WarpModel {None = 0, Translation, Similarity, Affine, Homography };
    //const WarpModel warp_model = Translation;
    //const WarpModel warp_model = Similarity;
    //const WarpModel warp_model = Affine;
    //const WarpModel warp_model = Homography; // default setting
};

// Check for Degeneracy, the higher level, the more strict check 
// at the price of more computations. 
namespace DegeneracyCheck{
    enum Level{ Level0 = 0, Level1, Level2, Level3 };
    //const  Level level = Level0;
    //const Level level = Level2; //default setting
    const bool check_homography = true;
};
// check if the lines are parallel.
namespace ParallelCheck{
    enum Level{ Level0 = 0, Level1};
    const Level level = Level0; //default setting
    const double cosanglethresh = 0.85;
    const double vertoffset = 2.0;
    const int ntrials = 100;
};

// Error messages
namespace RansacErrorCode{
    const int RANSAC_INSUFFICIENT_POINTS    = -1;
    const int RANSAC_WRONG_DIMENSIONS       = -2;
    const int RANSAC_NO_LS_SOLUTION         = -3;
    const int RANSAC_NO_NONDEGENERATE_DATA  = -4;
    const int RANSAC_NO_SOLUTION            = -5;
    const int RANSAC_INVALID_HOMOGRAPHY     = -6;
};

namespace RansacSetting{
    // Constants w.r.t. machine precision.
    const double    dbl_sqrt_2      = 1.41421356237309504880; //M_SqRT2;
    const double    dbl_eps_2       = std::numeric_limits<double>::epsilon();//DBL_EPSILON;
    const float     flt_eps_2       = std::numeric_limits<float>::epsilon();
    const double    det_eps_2       = 1E-3;
    const double    det_large_ratio = 100.0;
    const double    det_small_ratio = 0.01;

    // RANSAC settings
    const float     pconfidence         = 0.99f; // Desired probability of choosing at least one sample free from outliers
    const int       maxRansacTrials     = 10000; //Maximum number of trials before we give up.
    const int       minRansacTrials     = 1000; // Minimum number of trials before we give up.
    const int       maxDegeneracyTrials = 1000; //Max number of attempts to select a non-degenerate data set.
};

#endif
