
#include "OcclusionHandler.h"
#include "Utils/Utils.h"
#include "utils/ScanfillSkeleton.h"

const int occ_spatial_thr_x = 90;
const int occ_spatial_thr_y = 50;

OcclusionHandler::OcclusionHandler(int width, int height)
	:m_width(width), m_height(height)
{
	m_start_occ_frame_no = 0;
	m_end_occ_frame_no = 1000000;
	m_isObjectOccluded = false;
}

void OcclusionHandler::HandleSingleObjectOcclussion( int frame_count, 
													 ObjectModel& object_model, /* OUT */
													 ImgBgr &current_frame )
{
	int i;

	int width = current_frame.Width();
	int height = current_frame.Height();

	RegionAttributes& object_att = object_model.m_object_att;
	vector<Point>& contour_points = object_model.m_contour_points;
	vector<Shape>& contour_history = object_model.m_contour_history;
	vector<RegionAttributes>& fg_rgn_att = object_model.m_fg_rgn_att;

	printf("\n cen X = %d,cen Y = %d",(object_att.m_centroid.x),(object_att.m_centroid.y));
	printf("\n Velocity X = %lf,Velocity Y = %lf",(object_att.vel_x),(object_att.vel_y));
	
	if(!m_isObjectOccluded)
	{
		double curr_ObjSize = (double)object_att.m_pixel_list.size();
		double prev_ObjSize;	
		int t = frame_count-1,decay=0,occ_time=1;

		//check for last three frames if object size reduced lot to detect occlussion
		double area_thr = 0.6;
		for(i=0;i<occ_time;i++)
		{
			if(!contour_history[t].IsObjectOccluded){
			prev_ObjSize = (double)contour_history[t].size;
			prev_ObjSize = blepo_ex::Max(prev_ObjSize,1.0);

			if(curr_ObjSize/prev_ObjSize < area_thr)
				decay++;area_thr = area_thr - 0.1;
			}
			t--;
		}

		//if object is reducing more than 20 percent of its size in last three frames, then can say
		//it has occluded.
		//Problem always in differentiating deformation n occlussion

		if(decay == occ_time) 
		{
			TextDrawer td;
			td.DrawText(&current_frame,"Occlusion",object_att.m_centroid,Bgr(255,255,255));
			printf("Occlusion occurred\n");
			m_isObjectOccluded = true;
			MatchContour(frame_count-1, frame_count-7, contour_history, m_ShapeMatchFrameNo, m_ShapeIterateFrame);

			int dx=0,dy=0,top,left,bottom,right;
			ImgBinary mask;
			mask.Reset(width, height);
			Set(&mask,0);

			FillPolygon(&mask, contour_points);
			RegionAttributes temp_attr;
			object_model.UpdateObjectAttributes(mask, temp_attr, dx, dy);

			//printf("\n Occ - cen X = %d,cen Y = %d",(temp_attr.m_centroid.x),(temp_attr.m_centroid.y));
			//printf("\n Occ - Velocity X = %lf,Velocity Y = %lf",(temp_attr.avg_vel_x),(temp_attr.avg_vel_y));
			
			object_att.occ_vel_x = object_att.vel_x;
			object_att.occ_vel_y = object_att.vel_y;

			temp_attr.m_centroid.x += object_att.occ_vel_x;
			temp_attr.m_centroid.y += object_att.occ_vel_y;
			
			object_att.m_occ_centroid = temp_attr.m_centroid;

			printf("\n Just Occluded - cen X = %d,cen Y = %d",(object_att.m_occ_centroid.x),(object_att.m_occ_centroid.y));

			if((left=object_att.m_occ_centroid.x-occ_spatial_thr_x) <0)
				left=0;
			if((top=object_att.m_occ_centroid.y-occ_spatial_thr_y) <0)
				top=0;
			if((right=object_att.m_occ_centroid.x+occ_spatial_thr_x) >= width)
				right = width-1;
			if((bottom=object_att.m_occ_centroid.y+occ_spatial_thr_y) >= height)
				bottom = height-1;

			Set(&mask,Rect(left,top,right,bottom),1);
			object_model.UpdateObjectAttributes(mask, object_att, dx, dy);
			
			//_UpdateSpatialModel(m_fg_rgn_att, mask, dx, dy, true);
			for(i=0;i<fg_rgn_att.size();i++)
				object_model.CopySpatialModel(object_att, fg_rgn_att[i]);
		}
	}
	else //Object Already Occluded
	{
		int dx=0,dy=0,top,left,bottom,right;
		ImgBinary mask;
		mask.Reset(width, height);
		Set(&mask, 0);
		FillPolygon(&mask, m_occluded_contour_pts);
		RegionAttributes temp_attr;
		if(!object_model.UpdateObjectAttributes(mask, temp_attr, dx, dy))
			return;
		
		//compute new spatial based on object moving velocity in x n y
		object_att.m_occ_centroid.x += object_att.occ_vel_x; //temp_attr.m_centroid.x; //(object_att.avg_vel_x/m_frame_count);
		object_att.m_occ_centroid.y += object_att.occ_vel_y; //temp_attr.m_centroid.y; //(object_att.avg_vel_y/m_frame_count);

		printf("\n Occluded - Velocity X = %lf,Velocity Y = %lf",(object_att.occ_vel_x),(object_att.occ_vel_y));
		printf("\n Occluded - Centroid X = %d,Centroid Y = %d",(object_att.m_occ_centroid.x),(object_att.m_occ_centroid.y));

		Set(&mask,0);
		if((left=object_att.m_occ_centroid.x-occ_spatial_thr_x) <0)
			left=0;
		if((top=object_att.m_occ_centroid.y-occ_spatial_thr_y) <0)
			top=0;
		if((right=object_att.m_occ_centroid.x+occ_spatial_thr_x) >= width)
			right = width-1;
		if((bottom=object_att.m_occ_centroid.y+occ_spatial_thr_y) >= height)
			bottom = height-1;
		
		Set(&mask,Rect(left,top,right,bottom),1);
		object_model.UpdateObjectAttributes(mask, object_att, dx, dy);
		for(i=0;i<fg_rgn_att.size();i++)
			object_model.CopySpatialModel(object_att, fg_rgn_att[i]);
		//*****************************************

		double curr_ObjSize = temp_attr.m_pixel_list.size();
		curr_ObjSize = blepo_ex::Max(curr_ObjSize,1.0);
		double prev_ObjSize,prevmaxSize,ss;
		
		int t = frame_count-1, count=0;
		while(1)
		{
			//for object history which is non-occluded
			if(t>=0 && !contour_history[t].IsObjectOccluded)
			{
				prev_ObjSize = (double)contour_history[t].size;
				prevmaxSize = prev_ObjSize;
				
				//check if new size that has come is greater than last seen size
				if(prev_ObjSize/curr_ObjSize < 1.0) 
				{	
					//search for maximum size from last 10 frames where object is not occluded
					for(i=0;t>=0 && i<10;i++,t--){
						if(!contour_history[t].IsObjectOccluded){
						ss = (double)contour_history[t].size;
						if(ss>prevmaxSize)
							prevmaxSize = ss;
						}
					}
					
					//see if the size is atleast one-third.
					//if(prevmaxSize/curr_ObjSize < 3.0)
					{
						m_isObjectOccluded = false;
						printf("No Occlusion occurred\n");
						TextDrawer td;
						td.DrawText(&current_frame,"No-Occlusion",object_att.m_centroid,Bgr(255,255,255));
						contour_points = m_occluded_contour_pts;
						object_att = temp_attr;
						for(i=0;i<fg_rgn_att.size();i++)
							object_model.CopySpatialModel(object_att, fg_rgn_att[i]);
					}
				}
				break;
			}
			t--;
			
		}

	}

}

void OcclusionHandler::GetOccludedContourPoints( ObjectModel& object_model,
												 vector<Point> *occ_contourPts, 
												 ImgBgr &display_image )
{
	int i;static int k =0;
	vector<Point> destPts; 

	RegionAttributes& object_att = object_model.m_object_att;
	vector<Shape>& contour_history = object_model.m_contour_history;
	vector<Point>& contour_points = object_model.m_contour_points;
	
	if(m_ShapeIterateFrame >= contour_history.size())
			m_ShapeIterateFrame = m_ShapeMatchFrameNo;

	while(contour_history[m_ShapeIterateFrame].IsObjectOccluded)
	{
		m_ShapeIterateFrame++;
		if(m_ShapeIterateFrame >= contour_history.size())
			m_ShapeIterateFrame = m_ShapeMatchFrameNo;
	}

	vector<Point> srcPts = contour_history[m_ShapeIterateFrame].contour_points;
	//Point centroid = contour_history[m_ShapeMatchFrameNo].centroid;
	Point centroid = object_att.m_occ_centroid;

	int cx = centroid.x,new_cx;
	int cy = centroid.y,new_cy;
	
	//cx+=7;
	for(i=0;i<srcPts.size();i++){
		new_cx = cx+srcPts[i].x;
		new_cy = cy+srcPts[i].y;
		if(new_cx <0 || new_cy <0)
			assert("Error less than zero on contour point");
		if(new_cx<0)
			new_cx=0;
		if(new_cy<0)
			new_cy=0;
		if(new_cx >= m_width)
			new_cx = m_width-1;
		if(new_cy >= m_height)
			new_cy = m_height-1;

		destPts.push_back(Point(new_cx,new_cy));
	}
	
	//storing strength map contour points in occluded state
	m_occluded_contour_pts = contour_points;
	
	//storing contour points otained from best shape in m_contour_points
	*occ_contourPts = destPts;
	
	//Fill polygon n erode to obtain shape at lower scale
	/*int dx = 0, dy = 0;
	ImgBinary mask;ImgFloat fmask;
	mask.Reset(m_width, m_height);
	Set(&mask, 0);
	FillPolygon(&mask,*occ_contourPts);
	Erode3x3(mask,&mask);Convert(mask,&fmask);

	int mean_x = contour_history[m_ShapeIterateFrame].centroid.x;
	int mean_y = contour_history[m_ShapeIterateFrame].centroid.y;
	_WallFollow(fmask,occ_contourPts,mean_x,mean_y);*/

	//static Figure BestShapeWnd("Best Shape Match");
	//for(i=0; i<(*occ_contourPts).size(); i++) {
	//	DrawDot((*occ_contourPts)[i], &display_image, Bgr(255,255,255), 2);
	//}
	//BestShapeWnd.Draw(display_image);

	if(1) //would be required for moving object n stationary object
	 m_ShapeIterateFrame++;	  
}

void OcclusionHandler::MatchContour(int startSearchFrame,int endSearchFrame, const vector<Shape>& contour_history, int& shapeMatchFrameNo, int& shapeIterateFrame)
{
	int i,min_dist_frame;
	double distmeasure,d1,d2,min_distance;
	vector<Point> srcPts = contour_history[startSearchFrame].contour_points;
	vector<Point> destPts; 
	for(i=startSearchFrame-1;i>endSearchFrame;i--)
	{
		destPts = contour_history[i].contour_points;	
		Utils::HausdorffDistance(srcPts,destPts,&d1);
		Utils::HausdorffDistance(destPts,srcPts,&d2);
		distmeasure = blepo_ex::Max(d1,d2);
		if(i == startSearchFrame-1 || distmeasure < min_distance){
			min_distance = distmeasure;min_dist_frame = i;}
		destPts.clear();
	}
	shapeMatchFrameNo = min_dist_frame;
	shapeIterateFrame = shapeMatchFrameNo;
}
