
#ifndef _REGIONSEGMENTATION_H
#define _REGIONSEGMENTATION_H

#include "TheTracker.h"
#include "ObjectModel.h"
#include <list>

struct Density {
public:
	Density(Point pt, float density) : m_density(density), m_pt(pt) {}
	float m_density;
	Point m_pt;
};

struct RegionInfo {
public:
	list<Point> m_pixel_list;
	int m_total_area;
	int m_fg_area;
};

class RegionSegmentation {
private:
	int m_density_window;
	int m_current_label_no;
	double m_sigma_th;		// threshold to determine if the pixel belongs to a cluster. 
	int m_num_features;		// hard-coded as 3 in the constructor
	double m_area_th;		// The region's size should be greater than a minimum % of the image size. If not, the region is not considered
	double m_fg_total_ratio;
	double m_fg_mask_ratio;
	static ImgInt *m_labels;
	static list<RegionInfo> *m_regions;

	// Computes a density map
	void _ComputeDensityMap(const ImgBgr& image, const ImgBinary& mask, vector<Density> *density_list);

	// The following 4 fns segment the image based on the similarity of neighboring pixels. The seed points are chosen from the density map
	void _SegmentImage(const ImgBgr& image, const ImgBinary& input_mask);
	void _GrowRegion(const ImgBgr& image, const Point& seed_pt, const ImgBinary& input_mask, RegionInfo& rgn_info, ImgInt& labels);
	inline void _UpdateRunningSum(Bgr val, vector<int>& sum, vector<int>& sum_squares, int& size);
	inline bool _IsSame(Bgr val1, Bgr seed_color, const vector<int>& sum, const vector<int>& sum_squares, int size);

	// Removes all small regions
	void _RemoveSmallRegions(int image_size, list<RegionInfo> *regions, ImgInt& labels);

	// Classify the edge pixels to one of its nearest segment, if required. Should not be used for tracking.
	void _ClassifyEdgePixels(const ImgBgr& image, int width, int height);

	// Segments the region into foreground and background regions based on the input mask
	void _SegmentRegion(const ImgBinary& input_mask, vector<RegionAttributes> *fg_rgns, vector<RegionAttributes> *bg_rgns);
	
public:
	RegionSegmentation();
	void FreeMemory();
	void DoSegmentation(const ImgBgr& image, bool classify_edges, const ImgBinary& input_mask, double sigma, vector<RegionAttributes> *fg_rgns, vector<RegionAttributes> *bg_rgns);
	void SegmentSpecificRegions(const ImgBgr& image, const ImgBinary& mask, ImgInt& labels, vector<RegionAttributes> *regions);
};

#endif