// TheTracker.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "TheTracker.h"
#include "TrackerManager.h"
#include "Utils/IniWriter.h"
#include "Utils/IniReader.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

#define NClicks 10

int addObject(const ImgBgr& current_frame, TrackerManager& tracker_mgr, Figure& track_output_wnd, double seg_sigma, char* points_filename, char* fg_model_filename, char* bg_model_filename)
{

	Array<CPoint> cpoints;
	Array<Figure::MouseClick> mpoints;
	vector<Point> input_points;
	int object_id;

	track_output_wnd.Draw(current_frame);

	// Load/Grab the input points
	if(strlen(points_filename) == 0) {
		track_output_wnd.GrabLeftMouseClicks(&mpoints, 0x0000FF, 0xFF0000);				

		FILE *fp = fopen("monkey.txt", "w");
		for(int i=0;i<mpoints.Len();i++) {
			input_points.push_back(Point(mpoints[i].pt.x, mpoints[i].pt.y));
			fprintf(fp, "%d %d\n", mpoints[i].pt.x, mpoints[i].pt.y);
		}
		fclose(fp);
	}
	else {
		FILE *fp = fopen(points_filename, "r");
		int x, y;
		while(!feof(fp)) {
			fscanf(fp, "%d %d\n", &x, &y);
			input_points.push_back(Point(x,y));
		}
		fclose(fp);
	}

	// Initialize the tracker with the foreground and background model
	if(strlen(fg_model_filename) != 0 && strlen(bg_model_filename) != 0) {
		ImgGray tmp1, tmp2;
		ImgInt fg_label, bg_label;
		Load(fg_model_filename, &tmp1);
		Load(bg_model_filename, &tmp2);
		Convert(tmp1, &fg_label);
		Convert(tmp2, &bg_label);
		object_id = tracker_mgr.AddObject(current_frame, input_points, seg_sigma, &fg_label, &bg_label);
	}
	else
		object_id = tracker_mgr.AddObject(current_frame, input_points, seg_sigma);

	return object_id;
}


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
		if(argc < 2) {
			cout << "Please provide the config file as a command line argument" << endl;
			return 1;
		}

		int i;
		int status = 1, frame_count;
		int object_id = -1;
		ImgBgr current_frame;
		Figure track_output_wnd("**Tracker Output");

		// Load ini file..
		CIniReader iniReader(argv[1]);
		int num_objects = iniReader.ReadInteger("Tracker", "Num_Objects", 0);
		int max_num_frames = iniReader.ReadInteger("Tracker", "Max_Num_Frames", 999999);
		char *input_file = iniReader.ReadString("Tracker", "Input_File", "");
		if(strcmp(input_file, "") == 0) {
			cout << "No input file mentioned in the config" << endl;
			return 2;
		}
		double seg_sigma = iniReader.ReadFloat("Tracker", "Segmentation_Sigma", 3.0f);
		char **points_filename = NULL;
		char **fg_model_filename = NULL;
		char **bg_model_filename = NULL;
		char **current_spatial_filename = NULL;
		int *frame_no = NULL;

		// Load avi file
		AVIFrameReader avf;
		if(avf.Initialize(input_file,0) < 0) {
			cout << "Error loading avi file" << endl;
			return 3;
		}
		TrackerManager tracker_mgr(avf.GetWidth(), avf.GetHeight());
		bool flag = false;

		if(num_objects) {
			points_filename = new char*[num_objects] ;
			fg_model_filename = new char*[num_objects];
			bg_model_filename = new char*[num_objects];
			current_spatial_filename = new char*[num_objects];
			frame_no = new int[num_objects];
		}
		char group[100];
		for(i=0; i<num_objects; i++) {
			sprintf(group, "Object%d", i+1);
			points_filename[i] = iniReader.ReadString(group, "Points", "");
			fg_model_filename[i] = iniReader.ReadString(group, "Fg_Model", "");
			bg_model_filename[i] = iniReader.ReadString(group, "Bg_Model", "");
			frame_no[i] = iniReader.ReadInteger(group, "Frame_No", 0);
		}
		int unadded_objects = num_objects;

		if(avf.Initialized())
		{
			frame_count = avf.GetFrameCount();
			// Start tracking
			int frame_num = 0;
			while(status != 0)
			{
				int start_time = clock();
				status = avf.ReadNextFrame(&current_frame);

        if (status >= max_num_frames)  status = 0;

				if(unadded_objects) {
					for(int nobj=0; nobj<num_objects; nobj++) {
						if(frame_no[nobj] != frame_num) continue;
						addObject(current_frame, tracker_mgr, track_output_wnd, seg_sigma, points_filename[nobj], fg_model_filename[nobj], bg_model_filename[nobj]);
						unadded_objects--;
					} 
				}  // unadded objects

				if(unadded_objects == num_objects) {
					frame_num++;
					continue;
				}
				
				tracker_mgr.TrackNextFrame(current_frame, frame_num);
				ImgBgr display_image;
				display_image = current_frame;
				tracker_mgr.DrawContour(display_image);
				track_output_wnd.Draw(display_image);

				//char file_name[256];
				//sprintf(file_name, "C:\\clemson\\thesis\\Report\\Data\\Pets\\%03d.bmp",frame_num);
				//Save(display_image, file_name);

				frame_num++;
				int end_time = clock();
				cout << frame_num <<" / " << frame_count << ": " << end_time-start_time << " ms" << endl;	
//				EventLoop();  // uncomment this if you want to wait for user to hit a key after first iteration
			}  // status
		}   // avi initialization

    if (frame_count > max_num_frames)
    {
      cout << "Done.  Maximum number of frames reached." << endl;
    }
    else
    {
      cout << "Done." << endl;
    }

		if(num_objects) {
			for(i=0; i<num_objects; i++) {
				delete points_filename[i];
				delete fg_model_filename[i];
				delete bg_model_filename[i];
				delete current_spatial_filename[i];
			}
			delete [] points_filename;
			delete [] fg_model_filename;
			delete [] bg_model_filename;
			delete [] current_spatial_filename;
			delete [] frame_no;
		}
		EventLoop();

	}

	return nRetCode;
}


